﻿using Catamaran.Drawings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace Catamaran.CollectionGenericObjects
{
    public abstract class AbstractCompany
    {
        protected readonly int _placeSizeWidth = 180;

        protected readonly int _placeSizeHeight = 80;

        protected readonly int _pictureWidth;

        protected readonly int _pictureHeight;

        protected ICollectionGenericObjects<DrawingBoat>? _collection = null;

        private int GetMaxCount => (_pictureWidth / _placeSizeWidth) *  (_pictureHeight / _placeSizeHeight / 2) ;

        public AbstractCompany(int picWidth, int picHeight, ICollectionGenericObjects<DrawingBoat> collection) 
        {
            _pictureHeight = picHeight;
            _pictureWidth = picWidth;
            _collection = collection;
            _collection.SetMaxCount = GetMaxCount;
        }

        public static int operator +(AbstractCompany company, DrawingBoat boat)
        {
            return company._collection.Insert(boat);
        }

        public static DrawingBoat operator -(AbstractCompany company, int position) 
        {
            return company._collection.Remove(position);
        }

        public DrawingBoat? GetRandomObject()
        {
            Random rnd = new();
            return _collection?.Get(rnd.Next(GetMaxCount));
        }

        public Bitmap? Show()
        {
            Bitmap bitmap = new(_pictureWidth, _pictureHeight);
            Graphics graphics = Graphics.FromImage(bitmap);
            DrawBackground(graphics);
            SetObjectsPosition();
            for (int i = 0; i < (_collection?.Count ?? 0); ++i)
            {
                DrawingBoat? obj = _collection?.Get(i);
                obj?.DrawTransport(graphics);
            }
            return bitmap;
        }

        protected abstract void DrawBackground(Graphics g);

        protected abstract void SetObjectsPosition();
    }
}
