﻿using Catamaran.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Catamaran.Drawings
{
    public class DrawingBoat
    {
        public EntityBoat? EntityBoat { get; protected set; }

        private int? _pictureWidth;

        private int? _pictureHeight;

        protected int? _startPosX;

        protected int? _startPosY;

        private readonly int _drawingCatamaranWidth = 100;

        private readonly int _drawingCatamaranHeight = 40;

        public int? GetPosX => _startPosX;

        public int? GetPosY => _startPosY;

        public int GetWidth => _drawingCatamaranWidth;

        public int GetHeight => _drawingCatamaranHeight;



        private DrawingBoat()
        {
            _pictureHeight = null;
            _pictureWidth = null;
            _startPosX = null;
            _startPosY = null;
        }

        public DrawingBoat(int speed, double weight, Color bodyColor) : this()
        {
            EntityBoat = new EntityBoat(speed, weight, bodyColor);
            
        }

        protected DrawingBoat(int drawingCatamaranWidth, int drawingCatamaranHeight) : this()
        {
            _drawingCatamaranWidth = drawingCatamaranWidth;
            _drawingCatamaranHeight = drawingCatamaranHeight;
        }

        public bool SetPictureSize(int width, int height)
        {
            if (width > _drawingCatamaranWidth && height > _drawingCatamaranHeight)
            {
                _pictureWidth = width;
                _pictureHeight = height;
                if (_startPosX != null && _startPosY != null)
                {
                    if (_startPosX.Value < 0) _startPosX = 0;
                    if (_startPosY.Value < 0) _startPosY = 0;
                    if (_startPosX.Value + _drawingCatamaranWidth > _pictureWidth)
                    {
                        _startPosX = _pictureWidth - _drawingCatamaranWidth;
                    }
                    if (_startPosY.Value + _drawingCatamaranHeight > _pictureHeight)
                    {
                        _startPosY = _pictureHeight - _drawingCatamaranHeight;
                    }
                }

                return true;
            }
            return false;
        }

        public void SetPosition(int x, int y)
        {
            if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            else
            {
                _startPosX = x;
                _startPosY = y;
                if (_startPosX.Value < 0) _startPosX = 0;
                if (_startPosY.Value < 0) _startPosY = 0;
                if (_startPosX.Value + _drawingCatamaranWidth > _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawingCatamaranWidth;
                }
                if (_startPosY.Value + _drawingCatamaranHeight > _pictureHeight)
                {
                    _startPosY = _pictureHeight - _drawingCatamaranHeight;
                }
            }

        }

        public bool MoveTransport(DirectionType direction)
        {
            if (EntityBoat == null || !_startPosX.HasValue || !_startPosY.HasValue)
            {
                return false;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    if (_startPosX.Value - EntityBoat.Step > 0)
                    {
                        _startPosX -= (int)EntityBoat.Step;
                    }
                    return true;
                //вверх
                case DirectionType.Up:
                    if (_startPosY.Value - EntityBoat.Step > 0)
                    {
                        _startPosY -= (int)EntityBoat.Step;
                    }
                    return true;
                // вправо
                case DirectionType.Right:
                    if (_startPosX.Value + _drawingCatamaranWidth + EntityBoat.Step < _pictureWidth)
                    {
                        _startPosX += (int)EntityBoat.Step;
                    }
                    return true;
                //вниз
                case DirectionType.Down:
                    if (_startPosY.Value + _drawingCatamaranHeight + EntityBoat.Step < _pictureHeight)
                    {
                        _startPosY += (int)EntityBoat.Step;
                    }
                    return true;
                default:
                    return false;
            }

        }

        public virtual void DrawTransport(Graphics g)
        {
            if (EntityBoat == null || !_startPosX.HasValue || !_startPosY.HasValue)
            {
                return;
            }

            Pen pen = new(Color.Black);
            Brush baseBrush = new SolidBrush(EntityBoat.BodyColor);
            
            Brush brBrush = new SolidBrush(Color.Brown);

            // начало отрисовки


            Point[] body =
            {
                new Point(_startPosX.Value + 10, _startPosY.Value + 10),
                new Point(_startPosX.Value + 75, _startPosY.Value + 10),
                new Point(_startPosX.Value + 95, _startPosY.Value + 25),
                new Point(_startPosX.Value + 75, _startPosY.Value + 40),
                new Point(_startPosX.Value + 10, _startPosY.Value + 40),
                new Point(_startPosX.Value + 10, _startPosY.Value + 10)
            };
            g.FillPolygon(baseBrush, body);
            g.DrawPolygon(pen, body);

            g.FillEllipse(brBrush, _startPosX.Value + 15, _startPosY.Value + 15, 60, 20);
            g.DrawEllipse(pen, _startPosX.Value + 15, _startPosY.Value + 15, 60, 20);

            // конец отрисовки
        }
    }
}
