﻿using Catamaran.CollectionGenericObjects;
using Catamaran.Drawings;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Catamaran
{
    public partial class FormBoatColletion : Form
    {

        private AbstractCompany? _company = null;

        private readonly StorageCollection<DrawingBoat> _storageCollection;

        public FormBoatColletion()
        {
            InitializeComponent();
            _storageCollection = new();
        }

        private void comboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
        {
            panelCompanyTools.Enabled = false;
        }

        

        private void AddBoatButton_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }
            FormBoatConfig form = new();
            form.AddEvent(SetBoat);
            form.Show();
        }

        private void SetBoat(DrawingBoat boat)
        {
            if (_company == null || boat == null)
            {
                return;
            }

            if (_company + boat >= 0)
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }


        private static Color GetColor(Random random)
        {
            Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            return color;
        }

        private void DeleteButton_Click(object sender, EventArgs e)
        {
            if (_company == null || string.IsNullOrEmpty(maskedTextBox.Text))
            {
                return;
            }

            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }

            int pos = Convert.ToInt32(maskedTextBox.Text);
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        private void UpdateButton_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }
            pictureBox.Image = _company.Show();
        }

        private void GoToTestButton_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            DrawingBoat? boat = null;
            int counter = 100;
            while (boat == null)
            {
                boat = _company.GetRandomObject();

                counter--;
                if (counter <= 0) break;
            }

            if (boat == null)
            {
                return;
            }

            FormCatamaran form = new()
            {
                SetBoat = boat
            };
            form.ShowDialog();
        }

        private void RefreshListBoxItems()
        {
            listBoxCollection.Items.Clear();
            for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
            {
                string? colName = _storageCollection.Keys?[i];
                if (!string.IsNullOrEmpty(colName))
                {
                    listBoxCollection.Items.Add(colName);
                }
            }
        }

        private void buttonCollectionAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonArray.Checked && !radioButtonList.Checked))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            CollectionType collectionType = CollectionType.None;
            if (radioButtonArray.Checked)
            {
                collectionType = CollectionType.Array;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }

            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RefreshListBoxItems();
        }

        private void buttonCollectionDelete_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedItem == null || listBoxCollection.SelectedIndex < 0)
            {
                MessageBox.Show("Не выбрана коллекция");
                return;
            }
            if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            RefreshListBoxItems();
        }

        private void buttonCompanyCreate_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }

            ICollectionGenericObjects<DrawingBoat>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
            if (collection == null)
            {
                MessageBox.Show("Коллекция не инициализирована");
                return;
            }

            switch (comboBoxSelectorCompany.Text)
            {
                case "Хранилище":
                    _company = new Harbor(pictureBox.Width, pictureBox.Height, collection);
                    break;
            }

            panelCompanyTools.Enabled = true;
            RefreshListBoxItems();
        }

        
    }
}
