﻿using Catamaran.Drawings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Catamaran.MovementStrategy
{
    internal class MoveableBoat : IMoveableObject
    {
        private readonly DrawingBoat? _boat = null;

        public MoveableBoat(DrawingBoat boat)
        {
            _boat = boat;
        }

        public ObjectParameters? GetObjectPosition
        {
            get
            {
                if (_boat == null || _boat.EntityBoat == null || !_boat.GetPosX.HasValue || !_boat.GetPosY.HasValue)
                {
                    return null;
                }
                return new ObjectParameters(_boat.GetPosX.Value, _boat.GetPosY.Value, _boat.GetWidth, _boat.GetHeight);
            }
        }

        public int GetStep => (int)(_boat?.EntityBoat?.Step ?? 0);

        public bool TryMoveObject(MovementDirection direction)
        {
            if (_boat == null || _boat.EntityBoat == null)
            {
                return false;
            }

            return _boat.MoveTransport(GetDirectionType(direction));
        }

        private static DirectionType GetDirectionType(MovementDirection direction)
        {
            return direction switch
            {
                MovementDirection.Left => DirectionType.Left,
                MovementDirection.Right => DirectionType.Right,
                MovementDirection.Up => DirectionType.Up,
                MovementDirection.Down => DirectionType.Down,
                _ => DirectionType.Unknown
            };
        }
    }
}
